/*
		comparison functions for use with bsearch and qsort
		
    Copyright (C) 2015  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <timath.h>
#include <string.h>
#include "Num.h"

// COMPLEX ////////////////////////////////////////////////////////////////////

/* complex comparison (ascending) */
CALLBACK short complex_float_comp_asc(const void *a, 
	const void *b)
{
	return fcmp (Cabs(*(const _Complex float*)a), Cabs(*(const _Complex float*)b));
}

/* complex comparison (descending) */
CALLBACK short complex_float_comp_desc(const void *a, 
	const void *b)
{
	return -1 * complex_float_comp_asc(a, b);
}

// FLOAT //////////////////////////////////////////////////////////////////////

/* float comparison (ascending - also usable for longs) */
CALLBACK short float_comp_asc(const void *a, 
	const void *b)
{
	return fcmp (*(const float*)a, *(const float*)b);
}

/* float comparison (descending - also usable for longs) */
CALLBACK short float_comp_desc(const void *a, 
	const void *b)
{
	return -1 * float_comp_asc(a, b);
}

// STRING /////////////////////////////////////////////////////////////////////

/* string comparison (ascending) */
CALLBACK short string_comp_asc(const void *a, 
	const void *b)
{
	return strcmp (*(const char**)a, *(const char**)b);
}

/* string comparison (descending) */
CALLBACK short string_comp_desc(const void *a, 
	const void *b)
{
	return -1 * strcmp (*(const char**)a, *(const char**)b);
}
