/*
		Convolution and Deconvolution by FFT
		
    Copyright (C) 2015  Ralph Schenn

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
 
#include <alloc.h>
#include <error.h>
#include <estack.h>
#include <stdio.h>
#include "Stack.h" 
#include "FFT.h" 
#include "List.h" 
 
/* Convolution or Deconvolution by FFT
pointer to two lists of complex numbers on estack
convolution or deconvolution */
void ConDec(CESI argptr, 
	enum Bool inverse)
{
	HANDLE lst1Hdl = H_NULL;
	HANDLE lst2Hdl = H_NULL;

	TRY	
		// read lists into array
		unsigned short len1 = GetComplexFloatListArg (&argptr, &lst1Hdl);
						
		unsigned short len2 = GetComplexFloatListArg (&argptr, &lst2Hdl);
		
		CleanStack ();
				
		_Complex float *lst1Ptr = NULL;
		_Complex float *lst2Ptr = NULL;
		
		// list are not empty and same length
		if (lst1Hdl != H_NULL &&
			lst2Hdl != H_NULL)
		{
			if (len1 != len2) {
				ER_throw (ER_DIMENSION);
			}
			
			lst1Ptr = HeapDeref (lst1Hdl);
			lst2Ptr = HeapDeref (lst2Hdl);
		
			// perform forward transforms
			if (FFT (lst1Ptr, len1, 1) &&
				FFT (lst2Ptr, len2, 1)) {
				
				if (inverse) {
					// devide for deconvolution
					DivCompLstByLst(lst1Ptr, 
						lst2Ptr, 
						len1);
				} else {
					// multiply for convolution
					MulCompLstByLst(lst1Ptr, 
						lst2Ptr, 
						len1);
				}

				// perform inverse transform
				FFT (lst1Ptr, len1, -1);

				push_complex_float_list (lst1Ptr, len1);
			} else {
				ER_throw (ER_DOMAIN);
			}	
		} else {
			ER_throw (ER_DOMAIN);
		}
		
	FINALLY
		// avoid address error when arg is empty list
		if (lst1Hdl != H_NULL)
		{
			HeapFree (lst1Hdl);
		}

		if (lst2Hdl != H_NULL)
		{
			HeapFree (lst2Hdl);
		}
	ENDFINAL
}
