/*
		FFT Convolution
		
    Copyright (C) 2015  Ralph Schenn

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// ??? work in progress - do not use 
 
#include "FFT.h"
#include <math.h>
#include <alloc.h>
#include <error.h>
#include <estack.h>
#include "List.h" 
#include "Stack.h" 
 
/* FFT Convolution
argptr  - pointer to list of complex numbers on estack
inverse - inverse transform */
void FFTConv(CESI argptr, 
	enum Bool inverse)
{
	HANDLE handle1 = H_NULL;
	HANDLE handle2 = H_NULL;

	TRY	
		// read list into array
		unsigned short l1 = GetComplexFloatListArg (&argptr, &handle1);
		unsigned short l2 = GetComplexFloatListArg (&argptr, &handle2);
		
		CleanStack ();
		
		_Complex float *array1 = NULL;
		_Complex float *array2 = NULL;
		
		// list is not empty
		if (handle1 != H_NULL && handle2 != H_NULL)
		{
			array1 = HeapDeref (handle1);
			array2 = HeapDeref (handle2);
		
			// ??? pad the lists until they are power of two and same leghth
			
	  		if (!FFT( array1, l1, inverse) || !FFT( array2, l2, inverse))
	  		{
	  			ER_throw ( ER_DOMAIN );
	  		}
	  		
	  		if (inverse) {
	  			DivCompLstByLst(array1, array2, l1);
	  		} else {
	  			MulCompLstByLst(array1, array2, l1);
	  		}
	  		
			// push list in reverse order to estack
			push_complex_float_list (array1, l1); 			
	  	}
	FINALLY
		// avoid address error when arg is empty list
		if (handle1 != H_NULL)
		{
			HeapFree (handle1);
		}
		
		if (handle2 != H_NULL)
		{
			HeapFree (handle2);
		}
	ENDFINAL
}
