/*
		List
		
    Copyright (C) 2015  Ralph Schenn

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// THREE OPERANDS ////////////////////////////////////////////////////////////////////
 
/* add two complex lists */
void AddCompLstByLst3(_Complex float lst1[],
	_Complex float lst2[],
	_Complex float sum[],
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		sum[i] = lst1[i] + lst2[i];
	}
}
 
/* subtract two complex lists */
void SubCompLstByLst3(_Complex float lst1[],
	_Complex float lst2[],
	_Complex float dif[],
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		dif[i] = lst1[i] - lst2[i];
	}
}
 
/* multiply two complex lists */
void MulCompLstByLst3(_Complex float lst1[],
	_Complex float lst2[],
	_Complex float prod[],
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		prod[i] = lst1[i] * lst2[i];
	}
}
 
/* divide two complex lists */
void DivCompLstByLst3(_Complex float lst1[],
	_Complex float lst2[],
	_Complex float quot[],
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		quot[i] = lst1[i] / lst2[i];
	}
}
 
// TWO OPERANDS //////////////////////////////////////////////////////////////////////
 
/* add two complex lists */
void AddCompLstByLst(_Complex float lst1[], 
	_Complex float lst2[], 
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst1[i] += lst2[i];
	}
}
 
/* subtract two complex lists */
void SubCompLstByLst(_Complex float lst1[], 
	_Complex float lst2[], 
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst1[i] -= lst2[i];
	}
}
 
/* multiply two complex lists */
void MulCompLstByLst(_Complex float lst1[], 
	_Complex float lst2[], 
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst1[i] *= lst2[i];
	}
}
 
/* divide two complex lists */
void DivCompLstByLst(_Complex float lst1[], 
	_Complex float lst2[], 
	unsigned short length)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst1[i] /= lst2[i];
	}
}
 
// THREE OPERANDS ///////////////////////////////////////////////////////////////////
 
/* add scalar to complex list */
void AddCompLstBySca3(_Complex float lst[],
	_Complex float sum[],
	unsigned short length,
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		sum[i] = lst[i] + sca;
	}
}
 
/* subtract scalar from complex list */
void SubCompLstBySca3(_Complex float lst[],
	_Complex float dif[],
	unsigned short length,
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		dif[i] = lst[i] - sca;
	}
}
 
/* multiply complex list by scalar */
void MulCompLstBySca3(_Complex float lst[],
	_Complex float prod[],
	unsigned short length,
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		prod[i] = lst[i] * sca;
	}
}
 
/* divide complex list trough scalar */
void DivCompLstBySca3(_Complex float lst[],
	_Complex float quot[],
	unsigned short length,
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		quot[i] = lst[i] / sca;
	}
}
 
// TWO OPERANDS ////////////////////////////////////////////////////////////////////
 
/* add scalar to complex list */
void AddCompLstBySca(_Complex float lst[], 
	unsigned short length, 
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst[i] += sca;
	}
}
 
/* subtract scalar from complex list */
void SubCompLstBySca(_Complex float lst[], 
	unsigned short length, 
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst[i] -= sca;
	}
}
 
/* multiply complex list by scalar */
void MulCompLstBySca(_Complex float lst[], 
	unsigned short length, 
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst[i] *= sca;
	}
}
 
/* divide complex list trough scalar */
void DivCompLstBySca(_Complex float lst[], 
	unsigned short length, 
	_Complex float sca)
{
	unsigned short i;
 
	for (i = 0; i < length; i++) {
		lst[i] /= sca;
	}
}

// COMPLEX //////////////////////////////////////////////////////////
 
/* sum of complex list
lst - complex list
len - length of complex list */
_Complex float SumCompList(_Complex float lst[], 
	unsigned short len)
{
	unsigned short i;
	_Complex float s = 0;
 
	for (i = 0; i < len; i++) {
		s += lst[i];
	}
 
	return s;
}
 
/* product of complex list
lst - complex list
len - length of complex list */
_Complex float ProdCompList(_Complex float lst[], 
	unsigned short len)
{
	unsigned short i;
	_Complex float p = 1;
 
	for (i = 0; i < len; i++) {
		p *= lst[i];
	}
 
	return p;
}
 
// REAL /////////////////////////////////////////////////////////////
 
/* sum of real list
* lst - real list
* len - length of real list */
float SumList(float lst[], 
	unsigned short len)
{
	unsigned short i;
	float s = 0;
 
	for (i = 0; i < len; i++) {
		s += lst[i];
	}
 
	return s;
}
 
/* product of real list
lst - real list
len - length of real list */
float ProdList(float lst[], 
	unsigned short len)
{
	unsigned short i;
	float p = 1;
 
	for (i = 0; i < len; i++) {
		p *= lst[i];
	}
 
	return p;
}
