/*
		Moving Average
		
    Copyright (C) 2015  Ralph Schenn

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <alloc.h>
#include <asmtypes.h>
#include <error.h>
#include <estack.h>
#include <math.h>
#include <stdlib.h>
#include "Stack.h"

/* Moving Average
input signal
input signal length
average length */
BOOL MovAvg(float x[], 
	unsigned short xl, 
	unsigned short l,
	float y[])
{ 	
	unsigned short right = xl - l;
	unsigned short i;
	float s = 0;
 
	// length of average is not longer than length of list
	if (l <= xl) {
		// initial sum calculation
		for (i = 0; i < l; i++) {
			s += x[i];
		}
	 
		y[0] = s / l;  
	 
		// moving average
		for (i = 0; i < right; i++) {
			// add next input sample
			// subtract previous input sample
			s += x[i + l] - x[i];
			
			y[i + 1] = s / l;
		}
		
		return TRUE;
	}
	
	return FALSE; 
}

/* Moving Average for TI-89
pointer to list of real numbers on estack
length of moving average */
void MovAvgWrapper(CESI argptr,
	unsigned short lenAvg) {
	
	HANDLE xHdl = H_NULL;
	HANDLE yHdl = H_NULL;

	TRY	
		// read list into array
		unsigned short lenX = GetFloatListArg (&argptr, &xHdl);
				
		short lenY = lenX - lenAvg + 1;
				
		CleanStack ();
		
		float *xPtr = NULL;
		float *yPtr = NULL;
		
		// list is not empty
		if (xHdl != H_NULL)
		{
			xPtr = HeapDeref (xHdl);
			
			yHdl = HeapAlloc (lenY * sizeof(float));
			
			if (yHdl != H_NULL) {
	  			yPtr = HeapDeref (yHdl);
	  			
				if (MovAvg(xPtr, 
					lenX, 
					lenAvg,
					yPtr)) { 
					// push list in reverse order to estack
					push_float_list (yPtr, lenY);
				} else {
					ER_throw (ER_DOMAIN);
				}
			}			
	  	} else {
	  		push_Float (NAN);	
	  	}
	FINALLY
		// avoid address error when arg is empty list
		if (xHdl != H_NULL)
		{
			HeapFree (xHdl);
		}
		
		if (yHdl != H_NULL)
		{
			HeapFree (yHdl);
		}
	ENDFINAL
}
