/*
		binary search
		
    Copyright (C) 2013  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <alloc.h>
#include <args.h>
#include <asmtypes.h>
#include <error.h>
#include <stdlib.h>
#include <string.h>
#include <vat.h>
#include "Compare.h"
#include "Stack.h"

/* check if array of handles to strings contains string 
return -1 if string was not found 
when string is found the index is returned */
short ArrayContStr(HANDLE strArray[], 
	short length, 
	const char *str)
{
	const char *strPtr = NULL;
	
	// array has already been allocated
	if (strArray != NULL) {
		// loop over all strings
		while (--length >= 0) {
			strPtr = HeapDeref (strArray[length]);
			
			// string was found in array
			if (strcmp (str, strPtr) == 0) {
				return length;
			}
		}
	}
	
	return -1;
}

/* search memory block subblock in block 
this works for arbitrary content without the need for \0 string termination 
case sensitive search */ 
char *blockblock(const char *block, // ok
	const char *subBlock, 
	unsigned short blockLen, 
	unsigned short subBlockLen)
{
  unsigned short i, max = blockLen - subBlockLen;
  const char *ptr;
        
  // loop over MULTI_EXPR
  for (i = 0; i <= max; i++) {
    // starting address for block content comparison
    ptr = block + i;
                
    // test if subblock is found in block
    if (memcmp (ptr, subBlock, subBlockLen) == 0) {
      // subblock was found in block
    	return (char *)ptr;
    }
  }
        
  // subblock was not found in block
  return NULL;
}

/*char *blockblockci(const char *block,
	const char *subBlock, 
  unsigned short blockLen, 
  unsigned short subBlockLen)
// search memory block subblock in block 
this works for arbitrary content without the need for \0 string termination 
case insensitive search
{
	// ???
}*/

/* count number of occurences of val in array 
array must not be null-terminated */
unsigned short CountChar(char array[], 
	unsigned short length, 
	char val) // ok
{
	unsigned short i, count = 0;
	
	// loop over characters in array
	for (i = 0; i < length; i++) {
		// match was found
		if (array[i] == val) {
			count++;
		}
	}
	
	return count;
}

/* binary search a list of floats */
void SearchFloatList(CESI argptr)
{
	HANDLE handle = H_NULL;	

	TRY
		// read list into array
		unsigned short length = GetFloatListArg (&argptr, &handle);
		
		float value;
		
		if (GetArgType (argptr) == FLOAT_TAG) {
			value = GetFloatArg (argptr);
		} else {
			ER_throw (ER_DATATYPE);
		}
				
		CleanStack ();
		
		float *array = HeapDeref (handle);
		
		float *result = bsearch (&value, 
			array, 
			length, 
			sizeof(float), 
			float_comp_asc);
		
		// value was found in list
		if (result != NULL) {
			// convert pointer to index
			push_longint ((result - array) / sizeof(float) + 1); // ??? always returns 1
		// value was not found
		} else {
			push_longint (0);
		}
	FINALLY
		if (handle != H_NULL)
		{
			HeapFree (handle);			
		}
	ENDFINAL	
}
