/*
		sorting functions
		
    Copyright (C) 2013  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <alloc.h>
#include "Compare.h"
#include <error.h>
#include <estack.h>
#include "Sort.h"
#include "Stack.h"
#include <stdlib.h>
#include <string.h>

/* sort list of complex floats */
void SortComplexFloatList(CESI argptr, 
	order ord) // ok
{
	HANDLE handle = H_NULL;

	TRY	
		// read list into array
		unsigned short l = GetComplexFloatListArg (&argptr, &handle);
		
		CleanStack ();
		
		_Complex float *array = NULL;
		
		// list is not empty
		if (handle != H_NULL)
		{
			array = HeapDeref (handle);
		
			// sort the list
	  	qsort (array, l, sizeof(_Complex float), 
	  		ord == ASC ? complex_float_comp_asc : complex_float_comp_desc);
	  }
			
		// push list in reverse order to estack
		push_complex_float_list (array, l);
	FINALLY
		// avoid address error when arg is empty list
		if (handle != H_NULL)
		{
			HeapFree (handle);
		}
	ENDFINAL
}

/* sort list of floats */
void SortFloatList(CESI argptr, 
	order ord) // ok
{
	HANDLE handle = H_NULL;

	TRY
		// read list into array
		unsigned short l = GetFloatListArg (&argptr, &handle);			
						
		CleanStack ();
		
		float *array = NULL;
		
		// list is not empty
		if (handle != H_NULL)
		{
			array = HeapDeref (handle);
						
			// perform the sort
	  	qsort (array, l, sizeof(float), 
	  		ord == ASC ? float_comp_asc : float_comp_desc);		
	  }  	
			
		// push list in reverse order to estack
		push_float_list (array, l);
	FINALLY	
		// avoid address error when arg is empty list
		if (handle != H_NULL)
		{
			HeapFree (handle);
		}
	ENDFINAL
}

/* sort list of longs */
void SortLongList(CESI argptr, 
	order ord)
{
	HANDLE handle = H_NULL;

	TRY
		// read list into array
		unsigned short l = GetLongintListArg (&argptr, &handle);
					
		CleanStack ();
		
		long *array = NULL;
		
		// list is not empty
		if (handle != H_NULL)
		{
			array = HeapDeref (handle);
								
			// sort the list
	  	qsort (array, l, sizeof(long), 
	  		ord == ASC ? float_comp_asc : float_comp_desc);
	  }
			
		// push list in reverse order to estack
		push_longint_list (array, l);
	FINALLY
		// avoid address error when arg is empty list
		if (handle != H_NULL)
		{
			HeapFree (handle);
		}
	ENDFINAL
}

/* sort list of strings */
void SortStringList(CESI argptr, 
	order ord)
{
	HANDLE handle = H_NULL;	

	TRY
		// read list into array
		unsigned short l = GetStringListArg (&argptr, &handle);
		
		char **array = NULL;
		
		// list is not empty
		if (handle != H_NULL)
		{
			array = HeapDeref (handle);
		
			// sort the list
	  	qsort (array, l, sizeof(char *), 
	  		ord == ASC ? string_comp_asc : string_comp_desc);
	  }
			
		// push list in reverse order to estack
		push_string_list (array, l);		
	FINALLY
		// avoid address error when arg is empty list
		if (handle != H_NULL)
		{
			HeapFree (handle);
		}
	ENDFINAL
}
