/*
		convert tag decimal values to strings with tag names
		
    Copyright (C) 2013  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <limits.h>
#include <string.h>
#include <error.h>
#include <estack.h>
 
// tags
static const char *tags[] = {
	"VAR_NAME",     // 0
	"_VAR_Q",
	"VAR_R",
	"VAR_S",
	"VAR_T",
	"VAR_U",
	"VAR_V",
	"VAR_W",
	"VAR_X",
	"VAR_Y",
	"VAR_Z",
	"VAR_A",
	"VAR_B",
	"VAR_C",
	"VAR_D",
	"VAR_E",
	"VAR_F",
	"VAR_G",
	"VAR_H",
	"VAR_I",
	"VAR_J",
	"VAR_K",
	"VAR_L",
	"VAR_M",
	"VAR_N",
	"VAR_O",
	"VAR_P",
	"VAR_Q",
	"EXT_SYSTEM",
	"ARB_REAL",
	"ARB_INT",
	"POSINT",
	"NEGINT",
	"POSFRAC",
	"NEGFRAC",
	"FLOAT",
	"BCD",
	"PI",
	"EXP",
	"IM",
	"NEGINFINITY",
	"INFINITY",
	"PN_INFINITY",
	"UNDEF",
	"FALSE",
	"TRUE",
	"STR",
	"NOTHING",
	"ACOSH",
	"ASINH",
	"ATANH",
	"ASECH",
	"ACSCH",
	"ACOTH",
	"COSH",
	"SINH",
	"TANH",
	"SECH",
	"CSCH",
	"COTH",
	"ACOS",
	"ASIN",
	"ATAN",
	"ASEC",
	"ACSC",
	"ACOT",
	"RACOS",
	"RASIN",
	"RATAN",
	"COS",
	"SIN",
	"TAN",
	"SEC",
	"CSC",
	"COT",
	"ITAN",
	"ABS",
	"ANGLE",
	"CEILING",
	"FLOOR",
	"INT",
	"SIGN",
	"SQRT",
	"EXPF",
	"LN",
	"LOG",
	"FPART",
	"IPART",
	"CONJ",
	"IMAG",
	"REAL",
	"APPROX",
	"TEXPAND",
	"TCOLLECT",
	"GETDENOM",
	"GETNUM",
	"ERROR",
	"CUMSUM",
	"DET",
	"COLNORM",
	"ROWNORM",
	"NORM",
	"MEAN",
	"MEDIAN",
	"PRODUCT",
	"STDDEV",
	"SUM",
	"VARIANCE",
	"UNITV",
	"DIM",
	"MAT2LIST",
	"NEWLIST",
	"RREF",
	"REF",
	"IDENTITY",
	"DIAG",
	"COLDIM",
	"ROWDIM",
	"TRANSPOSE",
	"FACTORIAL",
	"PERCENT",
	"RADIANS",
	"NOT",
	"MINUS",
	"VEC_POLAR",
	"VEC_CYLIND",
	"VEC_SPHERE",
	"START",
	"ISTORE",
	"STORE",
	"WITH",
	"XOR",
	"OR",
	"AND",
	"LT",
	"LE",
	"EQ",
	"GE",
 	"GT",
	"NE",
	"ADD",
	"ADDELT",
	"SUB",
	"SUBELT",
	"MUL",
	"MULELT",
	"DIV",
	"DIVELT",
	"POW",
	"POWELT",
	"SINCOS",
	"SOLVE",
	"CSOLVE",
	"NSOLVE",
	"ZEROS",
	"CZEROS",
	"FMIN",
	"FMAX",
	"COMPLEX",
	"POLYEVAL",
	"RANDPOLY",
	"CROSSP",
	"DOTP",
	"GCD",
	"LCM",
	"MOD",
	"INTDIV",
	"REMAIN",
	"NCR",
	"NPR",
	"P2RX",
	"P2RY",
	"P2PTHETA",
	"P2PR",
	"AUGMENT",
	"NEWMAT",
	"RANDMAT",
	"SIMULT",
	"PART",
	"EXP2LIST",
	"RANDNORM",
	"MROW",
	"ROWADD",
	"ROWSWAP",
	"ARCLEN",
	"NINT",
	"PI_PRODUCT",
	"SIGMA_SUM",
	"MROWADD",
	"ANS",
	"ENTRY",
	"EXACT",
	"LOGB",
	"COMDENOM",
	"EXPAND",
	"FACTOR",
	"CFACTOR",
	"INTEGRATE",
	"DIFFERENTIATE",
	"AVGRC",
	"NDERIV",
	"TAYLOR",
	"LIMIT",
	"PROPFRAC",
	"WHEN",
	"ROUND",
	"DMS",
	"LEFT",
	"RIGHT",
	"MID",
	"SHIFT",
	"SEQ",
	"LIST2MAT",
	"SUBMAT",
	"SUBSCRIPT",
	"RAND",
	"MIN",
	"MAX",
	"LIST",
	"USERFUNC",
	"MATRIX",
	"FUNC",
	"DATA",
	"GDB",
	"PIC",
	"TEXT",
	"FIG",
	"MAC",
	"EXT",
	"EXT_INSTR",
	"END",
	"COMMENT",
	"NEXTEXPR",
	"NEWLINE",
	"ENDSTACK",
	"PN1",
	"PN2",
	"ERROR_MSG",
	"EIGVC",
	"EIGVL",
	"DASH",
	"LOCALVAR",
	"DESOLVE",
	"FDASH",
	"ASM",
	"ISPRIME",
	"OTH",
	"ROTATE" // 249
};
 
/* convert tag to string */
const char *Tag2Str(enum Tags tag)
{
	return tags[++tag];
}
 
#define MAX_TAG 249
 
/* convert string to tag */
enum Tags Str2Tag(const char *input)
{       
	unsigned char i;
        
	for (i = 0; i < MAX_TAG; i++) {
		// the string was found
		if (strcmp(tags[i], input) == 0) {
			return --i;
		}
	}
        
	// the string was not found
	ER_throw (ER_ARGUMENT);
}
 
// extended tags
static const char *extTags[]  = {
	"INDIR", // extTags[0] = 1
	"GETKEY",
	"GETFOLD",
	"SWITCH",
	"UNITCONV",
	"ORD",
	"EXPR",
	"CHAR",
	"STRING",
	"GETTYPE",
	"GETMODE",
	"SETFOLD",
	"PTTEST",
	"PXLTEST",
	"SETGRAPH",
	"SETTABLE",
	"SETMODE",
	"FORMAT",
	"INSTRING",
	"APPEND",
	"DD",
	"EXPR2DMS",
	"VEC2RECT",
	"VEC2POLAR",
	"VEC2CYLIND",
	"VEC2SPHERE",
	"PARENTH_START",
	"PARENTH_END"
	"MAT_START",
	"MAT_END",
	"LIST_START",
	"LIST_END",
	"COMMA",
	"SEMICOLON",
	"COMPLEX_ANGLE",
	"SINGLE_QUOTE",
	"QUOTE",
	"POLCPLX",
	"TMPCNV",
	"DELTA_TMPCNV",
	"GETUNITS",
	"SETUNITS",
	"BIN",
	"HEX",
	"INT2BIN",
	"INT2DEC",
	"INT2HEX",
	"DET_TOL",
	"REF_TOL",
	"RREF_TOL",
	"SIMULT_TOL",
	"GETCONFG",
	"V_AUGMENT",
	"MEAN_TWOARG",
	"PRODUCT_TWOARG",
	"STDDEV_TWOARG",
	"SUM_TWOARG",
	"VARIANCE_TWOARG",
	"DELTA_LIST", // extTag[58] = 59
	
	"ISCLKON", // extTag[59] = 70
	"GETDATE",
	"GETTIME",
	"GETTMZN",
	"SETDATE",
	"SETTIME",
	"SETTMZN",
	"DAYOFWK",
	"STARTTMR",
	"CHECKTMR",
	"TIMECNV",
	"GETDTFMT",
	"GETTMFMT",
	"GETDTSTR",
	"GETTMSTR",
	"SETDTFMT",
	"SETTMFMT",
	"ROOT", 
	"EXPRIO", 
	"IMPDIF", 
	"STDEVPOP", 
	"ISVAR", 
	"ISLOCKED", 
	"ISARCHIV", 
	"GRAD", 
	"TO_GRAD", 
	"TO_RAD",
	"TO_LN", 
	"TO_LOGBASE" // extTags[87] = 98
};
 
#define LOWER_MIN_VAL 1
#define LOWER_MAX_VAL 59
#define UPPER_MIN_VAL 70
#define UPPER_MAX_VAL 98
 
#define LOWER_MIN_IDX 0
#define LOWER_MAX_IDX 58
#define UPPER_MIN_IDX 59
#define UPPER_MAX_IDX 87
 
/* convert extended tag to string */
const char *ExtTag2Str(enum ExtTags input)
{
	// tag is falling in lower interval
	if (LOWER_MIN_VAL <= input && input <= LOWER_MAX_VAL) {
		return extTags[input - 1];
	// tag is falling in upper interval
	} else if (UPPER_MIN_VAL <= input && input <= UPPER_MAX_VAL) {
		// mind the gap between the intervals
		return extTags[input - 12];
	} else {
		ER_throw (ER_ARGUMENT);
	}
}
 
/* convert string to extended tag */
enum ExtTags Str2ExtTag(const char *input)
{
	short i;
                
	// upper values are not used for extended tags
	for (i = 0; i < UPPER_MAX_IDX; i++) {
		// the string was found
		if (strcmp(extTags[i], input) == 0) {
			// string is falling in lower interval
			if (LOWER_MIN_IDX <= i && i <= LOWER_MAX_IDX) {
				// enum is not 0-based
				return i + 1;
			// string is falling in upper interval
			} else if (UPPER_MIN_IDX <= i && i <= UPPER_MAX_IDX) {
				// observe the gap between the intervals
				return i + 12;  
			}
		}
	}
        
	// the string was not found
	ER_throw (ER_ARGUMENT);
}
 
 