/*
		RecVarSe - list variables with names containing string
		Example: RecVarSe("folder", "str")
		if the parameter "folder" is ommited matching variables in all 
		folders are listed
		
    Copyright (C) 2013  Ralph Schenn

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <error.h>
#include <string.h>
#include <vat.h>
#include "MultiExpr.h"
#include "Stack.h"

#define HOME "\x7F"
#define MAX_VAR_LENGTH 8
#define BUF_LENGTH 2 * MAX_VAR_LENGTH + 1

void _main(void)
{	
	// search folder name
	char folder[MAX_VAR_LENGTH];
	
	// search string
	char str[MAX_VAR_LENGTH];

	// flags for object search
	unsigned short searchFlags;

	// pointer to object
	SYM_ENTRY *symptr;

	// output buffer
	char out[BUF_LENGTH];

	// argument pointer
	CESI argptr = top_estack;
	
	// check number of arguments
	char numArgs = CheckNumArgsInt (1, 2);

	TRY
		// only search string was passed as argument				
		// search through all folders
		if (numArgs == 1) {
			// folder name
			//symName = NULL;
			
			// search all folders
			searchFlags = FO_RECURSE;
		// search only in  folder given in first argument
		} else {
			// get pointer to folder name from estack
			// (search string must be copied because we will clean the stack and
			// push list elements later overwriting the passed arguments)
			GetStrnArgCopy(&argptr, folder, MAX_VAR_LENGTH);
			
			// search only in specified folder
			searchFlags = FO_SINGLE_FOLDER;
		}
		
		// get search string from estack
		// (search string must be copied because we will clean the stack and
		// push list elements later overwriting the passed arguments)
		GetStrnArgCopy(&argptr, str, MAX_VAR_LENGTH);
	ONERR
		ER_throw (ER_DATATYPE);
	ENDTRY
		
	// lock home folder
	FolderOp (SYMSTR (HOME), FOP_LOCK);
	
	// find first object in VAT
	// (name of folder must be tagged format, not C format)
	symptr = SymFindFirst (SYMSTR (folder), searchFlags);
	
	// remove arguents from estack
	// (symptr points to the estack, so this should be done 
	// only after using this pointer)
	CleanStack ();

	// beginning of list
  push_END_TAG ();

	// at least one object exists in VAT
  if (symptr) {
	
    // loop over objects in VAT
    do {
			// list variables with matching names
      if (IsFile (symptr) && strstr(symptr->name, str)) {
				// concat the path
				sprintf (out, "%s\\%s", 
					// SymFindFolderName is not working when searching 
					// only the variables in one folder
					numArgs == 1 ? SymFindFolderName () : folder, 
					symptr->name);
				
				// add path to list
				push_ANSI_string (out);	
      }
        
      // get next object from VAT
    } while ((symptr = SymFindNext ()));
  }
  	
  // unlock home folder
  FolderOp (SYMSTR (HOME), FOP_UNLOCK);
  	
  // end of list
  push_LIST_TAG ();
}
