/*
		refs - find called user defined functions/programs in function/program
		Example: refs("folder\function")
		
    Copyright (C) 2013  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <alloc.h>
#include <args.h>
#include <error.h>
#include <vat.h>
#include "Memory.h"
#include "MultiExpr.h"
#include "Stack.h"
 
void _main(void)
{
  HANDLE refs = H_NULL;
  HANDLE *refPtr = NULL;
  unsigned short numRefs = 0;
	CESI argptr = top_estack;
        
  CheckNumArgs (1);
        
  if (GetArgNumType (0) != STR_TAG) {
  	ER_throw (ER_ARG_MUST_BE_STRING);
  }
        
  // get variable path from estack
  const char *path = GetStrnArg (argptr);
        
  CleanStack ();
        
  // search the variable with path
  HSym var = SymFind (SYMSTR (path));
        
  // variable was not found
  if (var.folder == 0) {
  	ER_throw (ER_DOMAIN);   
  }
    
  TRY
  	// get array of handle to strings with references
  	numRefs = GetReferences (&refs, DerefSym (var));
        
  	// get pointer to array of handles to references (strings)
  	refPtr = HeapDeref (refs);
 
 		// push references (path strings) to estack
  	push_string_list_handle (refPtr, numRefs);
  FINALLY
  	// free the handles to references (path strings)
  	StringHandleArrayFree (refPtr, numRefs);
  
  	// if at least one reference was found in variable
  	if (refs != H_NULL) {
  		// free the array of handles to paths
  		HeapFree (refs);
  	}
  ENDFINAL
}
