/*
		sort - sort lists of integers, floats, complex numbers or strings
		Example: sort({2,1,3}, "i", "a")
		("i"=int, "f"=float "c"=complex "s"=string
		"a"=ascending "d"=descending)
		list of floats may also contain integers
		list of complex numbers may also contain floats and integers
		
    Copyright (C) 2013  Ralph Schenn with support from Kevin Kofler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// ??? sort arbitrary expressions

#include <string.h>
#include <basfunc.h>
#include <error.h>
#include "Stack.h"
#include "Sort.h"

#define NUM_ARGS 3
#define LIST 0
#define TYPE 1
#define ORDER 2

// ??? what happens when passing empty list
// pass imaginary unit like this: cZeros(x^2+1,x)[2]
// pass -1 like this: zeros(x^2-1,x)[1]
void _main(void)
{	
	CESI argptr = top_estack;

	// check number of parameters
	CheckNumArgs (NUM_ARGS);
	
	// check type of first argument
	if (GetArgNumType (LIST) != LIST_TAG) {
		ER_throw (ER_ARGUMENT_MUST_BE_LIST);	
	}
		
	// check type of second and third argument
	if (GetArgNumType (TYPE) != STR_TAG || GetArgNumType (ORDER) != STR_TAG) {
		ER_throw (ER_ARG_MUST_BE_STRING);	
	}

	// get type argument		
	const char *type = GetStrnArgPos (TYPE);

	// third argument must be a(scending) or d(escending)
	char allowedOrder[] = { 'a', 'd' };
	const char *dir = CheckArgCharVal(ORDER, allowedOrder);
								
	switch (*type) {
		// list of complex numbers
		case 'c':
			SortComplexFloatList (argptr, dir[0]);
			break;
		
		// list of floats
		case 'f':
			SortFloatList (argptr, dir[0]);
			break;
		
		// list of integers
		case 'i':
			SortLongList (argptr, dir[0]);
			break;
		
		// list of strings
		// this is more difficult because we are working with
		// an array of pointers to strings without copying the
		// strings themselves. Because of this we must clean the 
		// stack after pushing the sorted list
		case 's': {
			ESI delete_begin = top_estack, delete_end = top_estack;
			
			// this changes only the parameter and not the global top_estack
			StackBegin (&delete_begin);
			
			// after sorting the array of pointers to strings references 
			// the unsorted strings in the list on the estack
			SortStringList (argptr, dir[0]);
			
			// remove unsorted list and direction from estack
			delete_between (delete_begin, delete_end);
			break;
		}
		default:
			CleanStack ();
			ER_throw (ER_DOMAIN);
			break;
	}
}
